
#include <time.h>
#include <sys/atom.h>
#include <string.h>

static NUTTIME __myTime;

// returns 1 if year is a leap year
//         0 otherwise
static int isLeapYear (u_short year)
{
	year += 1900;
	if (year % 4 == 0)
	{
		if (year % 100 == 0)
		{
			if (year % 400 == 0)
				return 1;
			else
				return 0;
		}
		else
			return 1;
	}
	else
		return 0;
}

// returns the number of days of a specific month
static u_char NumDays (u_char mon, u_short year)
{
	if (mon == 1)
		return (28 + isLeapYear (year));
	else 
		return ((mon % 7) & 0x01)? 30 : 31;
}

// __timer0_1s must be called from Timer 0 interrupt on a one second base.
void __timer0_1s(void)
{
	__myTime.tm_sec++;
	if (__myTime.tm_sec >= 60)
	{
		__myTime.tm_sec = 0;
		__myTime.tm_min++;
		if (__myTime.tm_min >= 60)
		{
			__myTime.tm_min = 0;
			__myTime.tm_hour++;
			if (__myTime.tm_hour >= 24)
			{
				__myTime.tm_hour = 0;
				__myTime.tm_mday++;
				__myTime.tm_yday++;
				__myTime.tm_wday = (__myTime.tm_wday+1)%7;
				if (__myTime.tm_mday > NumDays (__myTime.tm_mon, __myTime.tm_year))
				{
					__myTime.tm_mday = 1;
					__myTime.tm_mon++;
					if (__myTime.tm_mon >= 12)
					{
						__myTime.tm_mon = 0;
						__myTime.tm_year++;
						__myTime.tm_yday = 0;
					}
				}
			}
		}
	}
}

void NutGetTime (NUTTIME* theTime)
{
	NutEnterCritical();
	memcpy (theTime, &__myTime, sizeof(__myTime));
	NutExitCritical();	
}

// calculates the day of year in theTime. January, 1th is 0.
static void CalcYDay(NUTTIME* theTime)
{
	int sum = 0;
	int i;
	for (i = 0; i < theTime->tm_mon; i++)
		sum += NumDays (i, theTime->tm_year);
	theTime->tm_yday =  sum + theTime->tm_mday - 1;
}

// calculates the day of week in theTime.
// does not work with years smaller than 2000.
static void CalcWDay(NUTTIME* theTime)
{
	int sum = 6;		// 1.1.2000 was a saturday (6)
	int i;
	CalcYDay(theTime);
	for (i = 100; i < theTime->tm_year; i++)
		sum += 1 + isLeapYear(i);
	theTime->tm_wday = (theTime->tm_yday + sum) % 7;
}

int NutSetTime (NUTTIME* theTime)
{
	if (theTime->tm_sec >= 60)
		return -1;
		
	if (theTime->tm_min >= 60)
		return -2;
		
	if (theTime->tm_hour >= 24)
		return -3;

	if (theTime->tm_mon >= 12)
		return -5;
		
	if (theTime->tm_mday > NumDays (theTime->tm_mon, theTime->tm_year))
		return -4;
		
	CalcWDay(theTime);
	
	NutEnterCritical();
	memcpy (&__myTime, theTime, sizeof(__myTime));
	NutExitCritical();	
	return 0;
}
